/****************************************************************************
** Meta object code from reading C++ file 'reslist.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../qtgui/reslist.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'reslist.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7ResListE_t {};
} // unnamed namespace

template <> constexpr inline auto ResList::qt_create_metaobjectdata<qt_meta_tag_ZN7ResListE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ResList",
        "nextPageAvailable",
        "",
        "prevPageAvailable",
        "docPreviewClicked",
        "Rcl::Doc",
        "docSaveToFileClicked",
        "previewRequested",
        "showSnippets",
        "showSubDocs",
        "editRequested",
        "openWithRequested",
        "std::string",
        "cmd",
        "docExpand",
        "wordSelect",
        "wordReplace",
        "hasResults",
        "setDocSource",
        "std::shared_ptr<DocSequence>",
        "nsource",
        "resetList",
        "resPageUpOrBack",
        "resPageDownOrNext",
        "resultPageBack",
        "resultPageFirst",
        "resultPageNext",
        "resultPageFor",
        "docnum",
        "menuPreview",
        "menuSaveToFile",
        "menuEdit",
        "menuOpenWith",
        "QAction*",
        "menuCopyFN",
        "menuCopyPath",
        "menuCopyURL",
        "menuCopyText",
        "menuExpand",
        "menuPreviewParent",
        "menuOpenParent",
        "menuOpenFolder",
        "menuShowSnippets",
        "menuShowSubDocs",
        "previewExposed",
        "append",
        "text",
        "readDocSource",
        "highlighted",
        "link",
        "createPopupMenu",
        "QPoint",
        "pos",
        "showQueryDetails",
        "onUiPrefsChanged",
        "onLinkClicked",
        "QUrl",
        "onLinkClicked1",
        "onPopupJsDone",
        "QVariant",
        "languageChange",
        "setupArrows",
        "runStoredJS",
        "onPageScrollPositionChanged",
        "QPointF",
        "position",
        "onPageContentsSizeChanged",
        "QSizeF",
        "size"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'nextPageAvailable'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'prevPageAvailable'
        QtMocHelpers::SignalData<void(bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'docPreviewClicked'
        QtMocHelpers::SignalData<void(int, Rcl::Doc, int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { 0x80000000 | 5, 2 }, { QMetaType::Int, 2 },
        }}),
        // Signal 'docSaveToFileClicked'
        QtMocHelpers::SignalData<void(Rcl::Doc)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'previewRequested'
        QtMocHelpers::SignalData<void(Rcl::Doc)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'showSnippets'
        QtMocHelpers::SignalData<void(Rcl::Doc)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'showSubDocs'
        QtMocHelpers::SignalData<void(Rcl::Doc)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'editRequested'
        QtMocHelpers::SignalData<void(Rcl::Doc)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'openWithRequested'
        QtMocHelpers::SignalData<void(Rcl::Doc, std::string)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 }, { 0x80000000 | 12, 13 },
        }}),
        // Signal 'docExpand'
        QtMocHelpers::SignalData<void(Rcl::Doc)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'wordSelect'
        QtMocHelpers::SignalData<void(QString)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'wordReplace'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 }, { QMetaType::QString, 2 },
        }}),
        // Signal 'hasResults'
        QtMocHelpers::SignalData<void(int)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'setDocSource'
        QtMocHelpers::SlotData<void(std::shared_ptr<DocSequence>)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'resetList'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resPageUpOrBack'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resPageDownOrNext'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resultPageBack'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resultPageFirst'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resultPageNext'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resultPageFor'
        QtMocHelpers::SlotData<void(int)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 28 },
        }}),
        // Slot 'menuPreview'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuSaveToFile'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuEdit'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuOpenWith'
        QtMocHelpers::SlotData<void(QAction *)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 33, 2 },
        }}),
        // Slot 'menuCopyFN'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuCopyPath'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuCopyURL'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuCopyText'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuExpand'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuPreviewParent'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuOpenParent'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuOpenFolder'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuShowSnippets'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuShowSubDocs'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'previewExposed'
        QtMocHelpers::SlotData<void(int)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'append'
        QtMocHelpers::SlotData<void(const QString &)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 46 },
        }}),
        // Slot 'readDocSource'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'highlighted'
        QtMocHelpers::SlotData<void(const QString &)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 49 },
        }}),
        // Slot 'createPopupMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 51, 52 },
        }}),
        // Slot 'showQueryDetails'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onUiPrefsChanged'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onLinkClicked'
        QtMocHelpers::SlotData<void(const QUrl &)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 56, 2 },
        }}),
        // Slot 'onLinkClicked1'
        QtMocHelpers::SlotData<void(const QUrl &)>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 56, 2 },
        }}),
        // Slot 'onPopupJsDone'
        QtMocHelpers::SlotData<void(const QVariant &)>(58, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 59, 2 },
        }}),
        // Slot 'languageChange'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'setupArrows'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'runStoredJS'
        QtMocHelpers::SlotData<void(bool)>(62, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'onPageScrollPositionChanged'
        QtMocHelpers::SlotData<void(const QPointF &)>(63, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 64, 65 },
        }}),
        // Slot 'onPageContentsSizeChanged'
        QtMocHelpers::SlotData<void(const QSizeF &)>(66, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 67, 68 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ResList, qt_meta_tag_ZN7ResListE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ResList::staticMetaObject = { {
    QMetaObject::SuperData::link<QWebEngineView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7ResListE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7ResListE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7ResListE_t>.metaTypes,
    nullptr
} };

void ResList::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ResList *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->nextPageAvailable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->prevPageAvailable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->docPreviewClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 3: _t->docSaveToFileClicked((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1]))); break;
        case 4: _t->previewRequested((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1]))); break;
        case 5: _t->showSnippets((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1]))); break;
        case 6: _t->showSubDocs((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1]))); break;
        case 7: _t->editRequested((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1]))); break;
        case 8: _t->openWithRequested((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::string>>(_a[2]))); break;
        case 9: _t->docExpand((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1]))); break;
        case 10: _t->wordSelect((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->wordReplace((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 12: _t->hasResults((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->setDocSource((*reinterpret_cast<std::add_pointer_t<std::shared_ptr<DocSequence>>>(_a[1]))); break;
        case 14: _t->resetList(); break;
        case 15: _t->resPageUpOrBack(); break;
        case 16: _t->resPageDownOrNext(); break;
        case 17: _t->resultPageBack(); break;
        case 18: _t->resultPageFirst(); break;
        case 19: _t->resultPageNext(); break;
        case 20: _t->resultPageFor((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 21: _t->menuPreview(); break;
        case 22: _t->menuSaveToFile(); break;
        case 23: _t->menuEdit(); break;
        case 24: _t->menuOpenWith((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 25: _t->menuCopyFN(); break;
        case 26: _t->menuCopyPath(); break;
        case 27: _t->menuCopyURL(); break;
        case 28: _t->menuCopyText(); break;
        case 29: _t->menuExpand(); break;
        case 30: _t->menuPreviewParent(); break;
        case 31: _t->menuOpenParent(); break;
        case 32: _t->menuOpenFolder(); break;
        case 33: _t->menuShowSnippets(); break;
        case 34: _t->menuShowSubDocs(); break;
        case 35: _t->previewExposed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 36: _t->append((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 37: _t->readDocSource(); break;
        case 38: _t->highlighted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 39: _t->createPopupMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 40: _t->showQueryDetails(); break;
        case 41: _t->onUiPrefsChanged(); break;
        case 42: _t->onLinkClicked((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 43: _t->onLinkClicked1((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 44: _t->onPopupJsDone((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 45: _t->languageChange(); break;
        case 46: _t->setupArrows(); break;
        case 47: _t->runStoredJS((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 48: _t->onPageScrollPositionChanged((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 49: _t->onPageContentsSizeChanged((*reinterpret_cast<std::add_pointer_t<QSizeF>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ResList::*)(bool )>(_a, &ResList::nextPageAvailable, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResList::*)(bool )>(_a, &ResList::prevPageAvailable, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResList::*)(int , Rcl::Doc , int )>(_a, &ResList::docPreviewClicked, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResList::*)(Rcl::Doc )>(_a, &ResList::docSaveToFileClicked, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResList::*)(Rcl::Doc )>(_a, &ResList::previewRequested, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResList::*)(Rcl::Doc )>(_a, &ResList::showSnippets, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResList::*)(Rcl::Doc )>(_a, &ResList::showSubDocs, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResList::*)(Rcl::Doc )>(_a, &ResList::editRequested, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResList::*)(Rcl::Doc , std::string )>(_a, &ResList::openWithRequested, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResList::*)(Rcl::Doc )>(_a, &ResList::docExpand, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResList::*)(QString )>(_a, &ResList::wordSelect, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResList::*)(const QString & , const QString & )>(_a, &ResList::wordReplace, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResList::*)(int )>(_a, &ResList::hasResults, 12))
            return;
    }
}

const QMetaObject *ResList::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ResList::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7ResListE_t>.strings))
        return static_cast<void*>(this);
    return QWebEngineView::qt_metacast(_clname);
}

int ResList::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWebEngineView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 50)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 50;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 50)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 50;
    }
    return _id;
}

// SIGNAL 0
void ResList::nextPageAvailable(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void ResList::prevPageAvailable(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void ResList::docPreviewClicked(int _t1, Rcl::Doc _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void ResList::docSaveToFileClicked(Rcl::Doc _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void ResList::previewRequested(Rcl::Doc _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void ResList::showSnippets(Rcl::Doc _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void ResList::showSubDocs(Rcl::Doc _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void ResList::editRequested(Rcl::Doc _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void ResList::openWithRequested(Rcl::Doc _t1, std::string _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void ResList::docExpand(Rcl::Doc _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void ResList::wordSelect(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void ResList::wordReplace(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1, _t2);
}

// SIGNAL 12
void ResList::hasResults(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}
QT_WARNING_POP
