/********************************************************************************
** Form generated from reading UI file 'htmldialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_HTMLDIALOG_H
#define UI_HTMLDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_HtmlDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTextBrowser *browser;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *HtmlDialog)
    {
        if (HtmlDialog->objectName().isEmpty())
            HtmlDialog->setObjectName("HtmlDialog");
        HtmlDialog->resize(650, 650);
        verticalLayout = new QVBoxLayout(HtmlDialog);
        verticalLayout->setObjectName("verticalLayout");
        browser = new QTextBrowser(HtmlDialog);
        browser->setObjectName("browser");

        verticalLayout->addWidget(browser);

        buttonBox = new QDialogButtonBox(HtmlDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Ok);
        buttonBox->setCenterButtons(true);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(HtmlDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, HtmlDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, HtmlDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(HtmlDialog);
    } // setupUi

    void retranslateUi(QDialog *HtmlDialog)
    {
        HtmlDialog->setWindowTitle(QCoreApplication::translate("HtmlDialog", "Dialog", nullptr));
    } // retranslateUi

};

namespace Ui {
    class HtmlDialog: public Ui_HtmlDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_HTMLDIALOG_H
