/********************************************************************************
** Form generated from reading UI file 'actsearch.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ACTSEARCH_H
#define UI_ACTSEARCH_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ActSearchDLG
{
public:
    QHBoxLayout *horizontalLayout;
    QComboBox *actCMB;

    void setupUi(QDialog *ActSearchDLG)
    {
        if (ActSearchDLG->objectName().isEmpty())
            ActSearchDLG->setObjectName("ActSearchDLG");
        ActSearchDLG->resize(314, 43);
        horizontalLayout = new QHBoxLayout(ActSearchDLG);
        horizontalLayout->setObjectName("horizontalLayout");
        actCMB = new QComboBox(ActSearchDLG);
        actCMB->setObjectName("actCMB");
        actCMB->setEditable(true);
        actCMB->setMaxVisibleItems(15);
        actCMB->setInsertPolicy(QComboBox::NoInsert);
        actCMB->setFrame(false);

        horizontalLayout->addWidget(actCMB);


        retranslateUi(ActSearchDLG);

        QMetaObject::connectSlotsByName(ActSearchDLG);
    } // setupUi

    void retranslateUi(QDialog *ActSearchDLG)
    {
        ActSearchDLG->setWindowTitle(QCoreApplication::translate("ActSearchDLG", "Menu search", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ActSearchDLG: public Ui_ActSearchDLG {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ACTSEARCH_H
