/********************************************************************************
** Form generated from reading UI file 'firstidx.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FIRSTIDX_H
#define UI_FIRSTIDX_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCommandLinkButton>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_FirstIdxDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QCommandLinkButton *idxconfCLB;
    QCommandLinkButton *idxschedCLB;
    QPushButton *runidxPB;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *FirstIdxDialog)
    {
        if (FirstIdxDialog->objectName().isEmpty())
            FirstIdxDialog->setObjectName("FirstIdxDialog");
        FirstIdxDialog->resize(502, 503);
        verticalLayout_2 = new QVBoxLayout(FirstIdxDialog);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(FirstIdxDialog);
        label->setObjectName("label");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(1);
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);
        label->setTextFormat(Qt::RichText);
        label->setWordWrap(true);

        verticalLayout->addWidget(label);

        idxconfCLB = new QCommandLinkButton(FirstIdxDialog);
        idxconfCLB->setObjectName("idxconfCLB");
        idxconfCLB->setCheckable(false);
        idxconfCLB->setAutoDefault(false);

        verticalLayout->addWidget(idxconfCLB);

        idxschedCLB = new QCommandLinkButton(FirstIdxDialog);
        idxschedCLB->setObjectName("idxschedCLB");
        idxschedCLB->setAutoDefault(false);

        verticalLayout->addWidget(idxschedCLB);

        runidxPB = new QPushButton(FirstIdxDialog);
        runidxPB->setObjectName("runidxPB");

        verticalLayout->addWidget(runidxPB);


        verticalLayout_2->addLayout(verticalLayout);

        buttonBox = new QDialogButtonBox(FirstIdxDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout_2->addWidget(buttonBox);


        retranslateUi(FirstIdxDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, FirstIdxDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, FirstIdxDialog, qOverload<>(&QDialog::reject));
        QObject::connect(runidxPB, &QPushButton::clicked, FirstIdxDialog, qOverload<>(&QDialog::accept));

        QMetaObject::connectSlotsByName(FirstIdxDialog);
    } // setupUi

    void retranslateUi(QDialog *FirstIdxDialog)
    {
        FirstIdxDialog->setWindowTitle(QCoreApplication::translate("FirstIdxDialog", "First indexing setup", nullptr));
        label->setText(QCoreApplication::translate("FirstIdxDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">It appears that the index for this configuration does not exist.</span><br /><br />If you just want to index your home directory with a set of reasonable defaults, press the <span style=\" font-style:italic;\">Start indexing now</span> button. You will be able to adjust the details later. </p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0"
                        "px; -qt-block-indent:0; text-indent:0px;\">If you want more control, use the following links to adjust the indexing configuration and schedule.</p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">These tools can be accessed later from the <span style=\" font-style:italic;\">Preferences</span> menu.</p></body></html>", nullptr));
        idxconfCLB->setText(QCoreApplication::translate("FirstIdxDialog", "Indexing configuration", nullptr));
        idxconfCLB->setDescription(QCoreApplication::translate("FirstIdxDialog", "This will let you adjust the directories you want to index, and other parameters like excluded file paths or names, default character sets, etc.", nullptr));
        idxschedCLB->setText(QCoreApplication::translate("FirstIdxDialog", "Indexing schedule", nullptr));
        idxschedCLB->setDescription(QCoreApplication::translate("FirstIdxDialog", "This will let you chose between batch and real-time indexing, and set up an automatic  schedule for batch indexing (using cron).", nullptr));
        runidxPB->setText(QCoreApplication::translate("FirstIdxDialog", "Start indexing now", nullptr));
    } // retranslateUi

};

namespace Ui {
    class FirstIdxDialog: public Ui_FirstIdxDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FIRSTIDX_H
