/********************************************************************************
** Form generated from reading UI file 'webcache.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_WEBCACHE_H
#define UI_WEBCACHE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_Webcache
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *headerInfoLBL;
    QHBoxLayout *horizontalLayout;
    QLabel *searchLBL;
    QLineEdit *searchLE;
    QTableView *tableview;

    void setupUi(QDialog *Webcache)
    {
        if (Webcache->objectName().isEmpty())
            Webcache->setObjectName("Webcache");
        Webcache->resize(400, 300);
        verticalLayout = new QVBoxLayout(Webcache);
        verticalLayout->setObjectName("verticalLayout");
        headerInfoLBL = new QLabel(Webcache);
        headerInfoLBL->setObjectName("headerInfoLBL");

        verticalLayout->addWidget(headerInfoLBL);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        searchLBL = new QLabel(Webcache);
        searchLBL->setObjectName("searchLBL");

        horizontalLayout->addWidget(searchLBL);

        searchLE = new QLineEdit(Webcache);
        searchLE->setObjectName("searchLE");

        horizontalLayout->addWidget(searchLE);


        verticalLayout->addLayout(horizontalLayout);

        tableview = new QTableView(Webcache);
        tableview->setObjectName("tableview");
        tableview->setEditTriggers(QAbstractItemView::NoEditTriggers);
        tableview->horizontalHeader()->setStretchLastSection(true);
        tableview->verticalHeader()->setVisible(false);

        verticalLayout->addWidget(tableview);

#if QT_CONFIG(shortcut)
        searchLBL->setBuddy(searchLE);
#endif // QT_CONFIG(shortcut)

        retranslateUi(Webcache);

        QMetaObject::connectSlotsByName(Webcache);
    } // setupUi

    void retranslateUi(QDialog *Webcache)
    {
        Webcache->setWindowTitle(QCoreApplication::translate("Webcache", "Webcache editor", nullptr));
        headerInfoLBL->setText(QCoreApplication::translate("Webcache", "TextLabel", nullptr));
        searchLBL->setText(QCoreApplication::translate("Webcache", "Search regexp", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Webcache: public Ui_Webcache {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_WEBCACHE_H
