/****************************************************************************
** Meta object code from reading C++ file 'uiprefs_w.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../qtgui/uiprefs_w.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'uiprefs_w.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13UIPrefsDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto UIPrefsDialog::qt_create_metaobjectdata<qt_meta_tag_ZN13UIPrefsDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "UIPrefsDialog",
        "uiprefsDone",
        "",
        "showPTrans",
        "delPTrans",
        "showFontDialog",
        "resetReslistFont",
        "showStylesheetDialog",
        "resetStylesheet",
        "showSynFileDialog",
        "showSnipCssDialog",
        "resetSnipCss",
        "showViewAction",
        "mt",
        "addExtraDbPB_clicked",
        "delExtraDbPB_clicked",
        "togExtraDbPB_clicked",
        "showTrayIconCB_clicked",
        "actAllExtraDbPB_clicked",
        "unacAllExtraDbPB_clicked",
        "setStemLang",
        "lang",
        "editParaFormat",
        "editHeaderText",
        "extradDbSelectChanged",
        "extraDbEditPtrans",
        "resetShortcuts",
        "accept",
        "reject"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'uiprefsDone'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showPTrans'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'delPTrans'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showFontDialog'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resetReslistFont'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showStylesheetDialog'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resetStylesheet'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showSynFileDialog'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showSnipCssDialog'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resetSnipCss'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showViewAction'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showViewAction'
        QtMocHelpers::SlotData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'addExtraDbPB_clicked'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'delExtraDbPB_clicked'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'togExtraDbPB_clicked'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showTrayIconCB_clicked'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'actAllExtraDbPB_clicked'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'unacAllExtraDbPB_clicked'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setStemLang'
        QtMocHelpers::SlotData<void(const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Slot 'editParaFormat'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editHeaderText'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'extradDbSelectChanged'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'extraDbEditPtrans'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resetShortcuts'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'accept'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'reject'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<UIPrefsDialog, qt_meta_tag_ZN13UIPrefsDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject UIPrefsDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13UIPrefsDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13UIPrefsDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13UIPrefsDialogE_t>.metaTypes,
    nullptr
} };

void UIPrefsDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UIPrefsDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->uiprefsDone(); break;
        case 1: _t->showPTrans((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->delPTrans(); break;
        case 3: _t->showFontDialog(); break;
        case 4: _t->resetReslistFont(); break;
        case 5: _t->showStylesheetDialog(); break;
        case 6: _t->resetStylesheet(); break;
        case 7: _t->showSynFileDialog(); break;
        case 8: _t->showSnipCssDialog(); break;
        case 9: _t->resetSnipCss(); break;
        case 10: _t->showViewAction(); break;
        case 11: _t->showViewAction((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->addExtraDbPB_clicked(); break;
        case 13: _t->delExtraDbPB_clicked(); break;
        case 14: _t->togExtraDbPB_clicked(); break;
        case 15: _t->showTrayIconCB_clicked(); break;
        case 16: _t->actAllExtraDbPB_clicked(); break;
        case 17: _t->unacAllExtraDbPB_clicked(); break;
        case 18: _t->setStemLang((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 19: _t->editParaFormat(); break;
        case 20: _t->editHeaderText(); break;
        case 21: _t->extradDbSelectChanged(); break;
        case 22: _t->extraDbEditPtrans(); break;
        case 23: _t->resetShortcuts(); break;
        case 24: _t->accept(); break;
        case 25: _t->reject(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (UIPrefsDialog::*)()>(_a, &UIPrefsDialog::uiprefsDone, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (UIPrefsDialog::*)(const QString & )>(_a, &UIPrefsDialog::showPTrans, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (UIPrefsDialog::*)()>(_a, &UIPrefsDialog::delPTrans, 2))
            return;
    }
}

const QMetaObject *UIPrefsDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UIPrefsDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13UIPrefsDialogE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::uiPrefsDialogBase"))
        return static_cast< Ui::uiPrefsDialogBase*>(this);
    return QDialog::qt_metacast(_clname);
}

int UIPrefsDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 26;
    }
    return _id;
}

// SIGNAL 0
void UIPrefsDialog::uiprefsDone()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void UIPrefsDialog::showPTrans(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void UIPrefsDialog::delPTrans()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
