/****************************************************************************
** Meta object code from reading C++ file 'restable.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../qtgui/restable.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'restable.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11RecollModelE_t {};
} // unnamed namespace

template <> constexpr inline auto RecollModel::qt_create_metaobjectdata<qt_meta_tag_ZN11RecollModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "RecollModel",
        "sortDataChanged",
        "",
        "DocSeqSortSpec"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sortDataChanged'
        QtMocHelpers::SignalData<void(DocSeqSortSpec)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RecollModel, qt_meta_tag_ZN11RecollModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject RecollModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractTableModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11RecollModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11RecollModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11RecollModelE_t>.metaTypes,
    nullptr
} };

void RecollModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RecollModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sortDataChanged((*reinterpret_cast<std::add_pointer_t<DocSeqSortSpec>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RecollModel::*)(DocSeqSortSpec )>(_a, &RecollModel::sortDataChanged, 0))
            return;
    }
}

const QMetaObject *RecollModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RecollModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11RecollModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractTableModel::qt_metacast(_clname);
}

int RecollModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractTableModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void RecollModel::sortDataChanged(DocSeqSortSpec _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN18ResTableDetailAreaE_t {};
} // unnamed namespace

template <> constexpr inline auto ResTableDetailArea::qt_create_metaobjectdata<qt_meta_tag_ZN18ResTableDetailAreaE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ResTableDetailArea",
        "createPopupMenu",
        "",
        "QPoint",
        "pos",
        "setFont",
        "init"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'createPopupMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setFont'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'init'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ResTableDetailArea, qt_meta_tag_ZN18ResTableDetailAreaE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ResTableDetailArea::staticMetaObject = { {
    QMetaObject::SuperData::link<QTextBrowser::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18ResTableDetailAreaE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18ResTableDetailAreaE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18ResTableDetailAreaE_t>.metaTypes,
    nullptr
} };

void ResTableDetailArea::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ResTableDetailArea *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->createPopupMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 1: _t->setFont(); break;
        case 2: _t->init(); break;
        default: ;
        }
    }
}

const QMetaObject *ResTableDetailArea::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ResTableDetailArea::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18ResTableDetailAreaE_t>.strings))
        return static_cast<void*>(this);
    return QTextBrowser::qt_metacast(_clname);
}

int ResTableDetailArea::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTextBrowser::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN6SCDataE_t {};
} // unnamed namespace

template <> constexpr inline auto SCData::qt_create_metaobjectdata<qt_meta_tag_ZN6SCDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SCData",
        "activate",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'activate'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SCData, qt_meta_tag_ZN6SCDataE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SCData::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6SCDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6SCDataE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6SCDataE_t>.metaTypes,
    nullptr
} };

void SCData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SCData *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->activate(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *SCData::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SCData::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6SCDataE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int SCData::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN8ResTableE_t {};
} // unnamed namespace

template <> constexpr inline auto ResTable::qt_create_metaobjectdata<qt_meta_tag_ZN8ResTableE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ResTable",
        "docPreviewClicked",
        "",
        "Rcl::Doc",
        "docSaveToFileClicked",
        "previewRequested",
        "editRequested",
        "openWithRequested",
        "std::string",
        "cmd",
        "headerClicked",
        "docExpand",
        "showSubDocs",
        "showSnippets",
        "detailDocChanged",
        "std::shared_ptr<DocSequence>",
        "onTableView_currentChanged",
        "QModelIndex",
        "tableView_entered",
        "index",
        "setDocSource",
        "nsource",
        "saveColState",
        "resetSource",
        "readDocSource",
        "resetPos",
        "onSortDataChanged",
        "DocSeqSortSpec",
        "createPopupMenu",
        "QPoint",
        "pos",
        "onClicked",
        "onDoubleClick",
        "menuPreview",
        "menuSaveToFile",
        "menuSaveSelection",
        "menuEdit",
        "menuEditAndQuit",
        "menuOpenWith",
        "QAction*",
        "menuCopyFN",
        "menuCopyPath",
        "menuCopyURL",
        "menuCopyText",
        "menuCopyTextAndQuit",
        "menuExpand",
        "menuPreviewParent",
        "menuOpenParent",
        "menuOpenFolder",
        "menuShowSnippets",
        "menuShowSubDocs",
        "createHeaderPopupMenu",
        "deleteColumn",
        "addColumn",
        "resetSort",
        "saveAsCSV",
        "onLinkClicked",
        "QUrl",
        "makeRowVisible",
        "row",
        "takeFocus",
        "onUiPrefsChanged",
        "onNewShortcuts",
        "setCurrentRowFromKbd",
        "toggleHeader",
        "toggleVHeader"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'docPreviewClicked'
        QtMocHelpers::SignalData<void(int, Rcl::Doc, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { 0x80000000 | 3, 2 }, { QMetaType::Int, 2 },
        }}),
        // Signal 'docSaveToFileClicked'
        QtMocHelpers::SignalData<void(Rcl::Doc)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'previewRequested'
        QtMocHelpers::SignalData<void(Rcl::Doc)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'editRequested'
        QtMocHelpers::SignalData<void(Rcl::Doc)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'openWithRequested'
        QtMocHelpers::SignalData<void(Rcl::Doc, std::string)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 8, 9 },
        }}),
        // Signal 'headerClicked'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'docExpand'
        QtMocHelpers::SignalData<void(Rcl::Doc)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'showSubDocs'
        QtMocHelpers::SignalData<void(Rcl::Doc)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'showSnippets'
        QtMocHelpers::SignalData<void(Rcl::Doc)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'detailDocChanged'
        QtMocHelpers::SignalData<void(Rcl::Doc, std::shared_ptr<DocSequence>)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 15, 2 },
        }}),
        // Slot 'onTableView_currentChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 2 },
        }}),
        // Slot 'tableView_entered'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 19 },
        }}),
        // Slot 'setDocSource'
        QtMocHelpers::SlotData<void(std::shared_ptr<DocSequence>)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 21 },
        }}),
        // Slot 'saveColState'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resetSource'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'readDocSource'
        QtMocHelpers::SlotData<void(bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
        // Slot 'readDocSource'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'onSortDataChanged'
        QtMocHelpers::SlotData<void(DocSeqSortSpec)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 2 },
        }}),
        // Slot 'createPopupMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Slot 'onClicked'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 2 },
        }}),
        // Slot 'onDoubleClick'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 2 },
        }}),
        // Slot 'menuPreview'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuSaveToFile'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuSaveSelection'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuEdit'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuEditAndQuit'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuOpenWith'
        QtMocHelpers::SlotData<void(QAction *)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 39, 2 },
        }}),
        // Slot 'menuCopyFN'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuCopyPath'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuCopyURL'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuCopyText'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuCopyTextAndQuit'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuExpand'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuPreviewParent'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuOpenParent'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuOpenFolder'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuShowSnippets'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuShowSubDocs'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'createHeaderPopupMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 2 },
        }}),
        // Slot 'deleteColumn'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addColumn'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resetSort'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveAsCSV'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onLinkClicked'
        QtMocHelpers::SlotData<void(const QUrl &)>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 57, 2 },
        }}),
        // Slot 'makeRowVisible'
        QtMocHelpers::SlotData<void(int)>(58, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 59 },
        }}),
        // Slot 'takeFocus'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onUiPrefsChanged'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onNewShortcuts'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setCurrentRowFromKbd'
        QtMocHelpers::SlotData<void(int)>(63, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 59 },
        }}),
        // Slot 'toggleHeader'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleVHeader'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ResTable, qt_meta_tag_ZN8ResTableE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ResTable::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8ResTableE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8ResTableE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8ResTableE_t>.metaTypes,
    nullptr
} };

void ResTable::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ResTable *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->docPreviewClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 1: _t->docSaveToFileClicked((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1]))); break;
        case 2: _t->previewRequested((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1]))); break;
        case 3: _t->editRequested((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1]))); break;
        case 4: _t->openWithRequested((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::string>>(_a[2]))); break;
        case 5: _t->headerClicked(); break;
        case 6: _t->docExpand((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1]))); break;
        case 7: _t->showSubDocs((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1]))); break;
        case 8: _t->showSnippets((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1]))); break;
        case 9: _t->detailDocChanged((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::shared_ptr<DocSequence>>>(_a[2]))); break;
        case 10: _t->onTableView_currentChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 11: _t->tableView_entered((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 12: _t->setDocSource((*reinterpret_cast<std::add_pointer_t<std::shared_ptr<DocSequence>>>(_a[1]))); break;
        case 13: _t->saveColState(); break;
        case 14: _t->resetSource(); break;
        case 15: _t->readDocSource((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->readDocSource(); break;
        case 17: _t->onSortDataChanged((*reinterpret_cast<std::add_pointer_t<DocSeqSortSpec>>(_a[1]))); break;
        case 18: _t->createPopupMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 19: _t->onClicked((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 20: _t->onDoubleClick((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 21: _t->menuPreview(); break;
        case 22: _t->menuSaveToFile(); break;
        case 23: _t->menuSaveSelection(); break;
        case 24: _t->menuEdit(); break;
        case 25: _t->menuEditAndQuit(); break;
        case 26: _t->menuOpenWith((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 27: _t->menuCopyFN(); break;
        case 28: _t->menuCopyPath(); break;
        case 29: _t->menuCopyURL(); break;
        case 30: _t->menuCopyText(); break;
        case 31: _t->menuCopyTextAndQuit(); break;
        case 32: _t->menuExpand(); break;
        case 33: _t->menuPreviewParent(); break;
        case 34: _t->menuOpenParent(); break;
        case 35: _t->menuOpenFolder(); break;
        case 36: _t->menuShowSnippets(); break;
        case 37: _t->menuShowSubDocs(); break;
        case 38: _t->createHeaderPopupMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 39: _t->deleteColumn(); break;
        case 40: _t->addColumn(); break;
        case 41: _t->resetSort(); break;
        case 42: _t->saveAsCSV(); break;
        case 43: _t->onLinkClicked((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 44: _t->makeRowVisible((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 45: _t->takeFocus(); break;
        case 46: _t->onUiPrefsChanged(); break;
        case 47: _t->onNewShortcuts(); break;
        case 48: _t->setCurrentRowFromKbd((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 49: _t->toggleHeader(); break;
        case 50: _t->toggleVHeader(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ResTable::*)(int , Rcl::Doc , int )>(_a, &ResTable::docPreviewClicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResTable::*)(Rcl::Doc )>(_a, &ResTable::docSaveToFileClicked, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResTable::*)(Rcl::Doc )>(_a, &ResTable::previewRequested, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResTable::*)(Rcl::Doc )>(_a, &ResTable::editRequested, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResTable::*)(Rcl::Doc , std::string )>(_a, &ResTable::openWithRequested, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResTable::*)()>(_a, &ResTable::headerClicked, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResTable::*)(Rcl::Doc )>(_a, &ResTable::docExpand, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResTable::*)(Rcl::Doc )>(_a, &ResTable::showSubDocs, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResTable::*)(Rcl::Doc )>(_a, &ResTable::showSnippets, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResTable::*)(Rcl::Doc , std::shared_ptr<DocSequence> )>(_a, &ResTable::detailDocChanged, 9))
            return;
    }
}

const QMetaObject *ResTable::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ResTable::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8ResTableE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::ResTable"))
        return static_cast< Ui::ResTable*>(this);
    return QWidget::qt_metacast(_clname);
}

int ResTable::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 51)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 51;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 51)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 51;
    }
    return _id;
}

// SIGNAL 0
void ResTable::docPreviewClicked(int _t1, Rcl::Doc _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void ResTable::docSaveToFileClicked(Rcl::Doc _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void ResTable::previewRequested(Rcl::Doc _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void ResTable::editRequested(Rcl::Doc _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void ResTable::openWithRequested(Rcl::Doc _t1, std::string _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void ResTable::headerClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void ResTable::docExpand(Rcl::Doc _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void ResTable::showSubDocs(Rcl::Doc _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void ResTable::showSnippets(Rcl::Doc _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void ResTable::detailDocChanged(Rcl::Doc _t1, std::shared_ptr<DocSequence> _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}
QT_WARNING_POP
