/****************************************************************************
** Meta object code from reading C++ file 'snippets_w.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../qtgui/snippets_w.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'snippets_w.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9SnippetsWE_t {};
} // unnamed namespace

template <> constexpr inline auto SnippetsW::qt_create_metaobjectdata<qt_meta_tag_ZN9SnippetsWE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SnippetsW",
        "startNativeViewer",
        "",
        "Rcl::Doc",
        "pagenum",
        "term",
        "line",
        "zoomIn",
        "zoomOut",
        "onLinkClicked",
        "QUrl",
        "onSetDoc",
        "doc",
        "std::shared_ptr<DocSequence>",
        "source",
        "createPopupMenu",
        "QPoint",
        "pos",
        "onNewShortcuts",
        "slotEditFind",
        "slotEditFindNext",
        "slotEditFindPrevious",
        "slotSearchTextChanged",
        "slotZoomIn",
        "slotZoomOut",
        "reloadByRelevance",
        "reloadByPage",
        "onUiPrefsChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'startNativeViewer'
        QtMocHelpers::SignalData<void(Rcl::Doc, int, QString, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { QMetaType::Int, 4 }, { QMetaType::QString, 5 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'zoomIn'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'zoomOut'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onLinkClicked'
        QtMocHelpers::SlotData<void(const QUrl &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Slot 'onSetDoc'
        QtMocHelpers::SlotData<void(Rcl::Doc, std::shared_ptr<DocSequence>)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 12 }, { 0x80000000 | 13, 14 },
        }}),
        // Slot 'createPopupMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'onNewShortcuts'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEditFind'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotEditFindNext'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotEditFindPrevious'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSearchTextChanged'
        QtMocHelpers::SlotData<void(const QString &)>(22, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'slotZoomIn'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotZoomOut'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'reloadByRelevance'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'reloadByPage'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onUiPrefsChanged'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SnippetsW, qt_meta_tag_ZN9SnippetsWE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SnippetsW::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9SnippetsWE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9SnippetsWE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9SnippetsWE_t>.metaTypes,
    nullptr
} };

void SnippetsW::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SnippetsW *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->startNativeViewer((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 1: _t->zoomIn(); break;
        case 2: _t->zoomOut(); break;
        case 3: _t->onLinkClicked((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 4: _t->onSetDoc((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::shared_ptr<DocSequence>>>(_a[2]))); break;
        case 5: _t->createPopupMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 6: _t->onNewShortcuts(); break;
        case 7: _t->slotEditFind(); break;
        case 8: _t->slotEditFindNext(); break;
        case 9: _t->slotEditFindPrevious(); break;
        case 10: _t->slotSearchTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->slotZoomIn(); break;
        case 12: _t->slotZoomOut(); break;
        case 13: _t->reloadByRelevance(); break;
        case 14: _t->reloadByPage(); break;
        case 15: _t->onUiPrefsChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SnippetsW::*)(Rcl::Doc , int , QString , int )>(_a, &SnippetsW::startNativeViewer, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SnippetsW::*)()>(_a, &SnippetsW::zoomIn, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SnippetsW::*)()>(_a, &SnippetsW::zoomOut, 2))
            return;
    }
}

const QMetaObject *SnippetsW::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SnippetsW::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9SnippetsWE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::Snippets"))
        return static_cast< Ui::Snippets*>(this);
    return QWidget::qt_metacast(_clname);
}

int SnippetsW::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void SnippetsW::startNativeViewer(Rcl::Doc _t1, int _t2, QString _t3, int _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 1
void SnippetsW::zoomIn()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void SnippetsW::zoomOut()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
namespace {
struct qt_meta_tag_ZN11SnipWebPageE_t {};
} // unnamed namespace

template <> constexpr inline auto SnipWebPage::qt_create_metaobjectdata<qt_meta_tag_ZN11SnipWebPageE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SnipWebPage"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SnipWebPage, qt_meta_tag_ZN11SnipWebPageE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SnipWebPage::staticMetaObject = { {
    QMetaObject::SuperData::link<QWebEnginePage::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11SnipWebPageE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11SnipWebPageE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11SnipWebPageE_t>.metaTypes,
    nullptr
} };

void SnipWebPage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SnipWebPage *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *SnipWebPage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SnipWebPage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11SnipWebPageE_t>.strings))
        return static_cast<void*>(this);
    return QWebEnginePage::qt_metacast(_clname);
}

int SnipWebPage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWebEnginePage::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
