/********************************************************************************
** Form generated from reading UI file 'spell.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SPELL_H
#define UI_SPELL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_SpellBase
{
public:
    QVBoxLayout *verticalLayout_2;
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QComboBox *expTypeCMB;
    QLineEdit *baseWordLE;
    QLabel *label;
    QCheckBox *caseSensCB;
    QCheckBox *diacSensCB;
    QComboBox *stemLangCMB;
    QPushButton *expandPB;
    QPushButton *dismissPB;
    QLabel *statsLBL;
    QTableWidget *resTW;

    void setupUi(QWidget *SpellBase)
    {
        if (SpellBase->objectName().isEmpty())
            SpellBase->setObjectName("SpellBase");
        SpellBase->resize(520, 465);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(SpellBase->sizePolicy().hasHeightForWidth());
        SpellBase->setSizePolicy(sizePolicy);
        SpellBase->setMinimumSize(QSize(100, 100));
        verticalLayout_2 = new QVBoxLayout(SpellBase);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(6);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        expTypeCMB = new QComboBox(SpellBase);
        expTypeCMB->setObjectName("expTypeCMB");

        gridLayout->addWidget(expTypeCMB, 0, 0, 1, 1);

        baseWordLE = new QLineEdit(SpellBase);
        baseWordLE->setObjectName("baseWordLE");
        baseWordLE->setMinimumSize(QSize(100, 0));

        gridLayout->addWidget(baseWordLE, 0, 1, 1, 2);

        label = new QLabel(SpellBase);
        label->setObjectName("label");

        gridLayout->addWidget(label, 1, 0, 1, 1);

        caseSensCB = new QCheckBox(SpellBase);
        caseSensCB->setObjectName("caseSensCB");

        gridLayout->addWidget(caseSensCB, 1, 1, 1, 1);

        diacSensCB = new QCheckBox(SpellBase);
        diacSensCB->setObjectName("diacSensCB");

        gridLayout->addWidget(diacSensCB, 1, 2, 1, 1);

        stemLangCMB = new QComboBox(SpellBase);
        stemLangCMB->setObjectName("stemLangCMB");

        gridLayout->addWidget(stemLangCMB, 2, 0, 1, 1);

        expandPB = new QPushButton(SpellBase);
        expandPB->setObjectName("expandPB");
        expandPB->setEnabled(false);
        expandPB->setFocusPolicy(Qt::NoFocus);

        gridLayout->addWidget(expandPB, 2, 1, 1, 1);

        dismissPB = new QPushButton(SpellBase);
        dismissPB->setObjectName("dismissPB");
        dismissPB->setEnabled(true);
        dismissPB->setFocusPolicy(Qt::NoFocus);

        gridLayout->addWidget(dismissPB, 2, 2, 1, 1);


        verticalLayout->addLayout(gridLayout);

        statsLBL = new QLabel(SpellBase);
        statsLBL->setObjectName("statsLBL");
        QFont font;
        font.setPointSize(7);
        statsLBL->setFont(font);

        verticalLayout->addWidget(statsLBL);

        resTW = new QTableWidget(SpellBase);
        if (resTW->columnCount() < 2)
            resTW->setColumnCount(2);
        if (resTW->rowCount() < 2)
            resTW->setRowCount(2);
        resTW->setObjectName("resTW");
        resTW->setRowCount(2);
        resTW->setColumnCount(2);
        resTW->horizontalHeader()->setStretchLastSection(false);
        resTW->verticalHeader()->setVisible(false);

        verticalLayout->addWidget(resTW);


        verticalLayout_2->addLayout(verticalLayout);

        QWidget::setTabOrder(baseWordLE, expandPB);
        QWidget::setTabOrder(expandPB, dismissPB);
        QWidget::setTabOrder(dismissPB, stemLangCMB);

        retranslateUi(SpellBase);

        QMetaObject::connectSlotsByName(SpellBase);
    } // setupUi

    void retranslateUi(QWidget *SpellBase)
    {
        SpellBase->setWindowTitle(QCoreApplication::translate("SpellBase", "Term Explorer", nullptr));
        label->setText(QCoreApplication::translate("SpellBase", "Match", nullptr));
        caseSensCB->setText(QCoreApplication::translate("SpellBase", "Case", nullptr));
        diacSensCB->setText(QCoreApplication::translate("SpellBase", "Accents", nullptr));
        expandPB->setText(QCoreApplication::translate("SpellBase", "&Expand ", nullptr));
#if QT_CONFIG(shortcut)
        expandPB->setShortcut(QCoreApplication::translate("SpellBase", "Alt+E", nullptr));
#endif // QT_CONFIG(shortcut)
        dismissPB->setText(QCoreApplication::translate("SpellBase", "&Close", nullptr));
#if QT_CONFIG(shortcut)
        dismissPB->setShortcut(QCoreApplication::translate("SpellBase", "Alt+C", nullptr));
#endif // QT_CONFIG(shortcut)
        statsLBL->setText(QCoreApplication::translate("SpellBase", "No db info.", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SpellBase: public Ui_SpellBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SPELL_H
