/********************************************************************************
** Form generated from reading UI file 'snippets.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SNIPPETS_H
#define UI_SNIPPETS_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Snippets
{
public:
    QVBoxLayout *verticalLayout_2;
    QVBoxLayout *verticalLayout;
    QWidget *browserw;
    QHBoxLayout *horizontalLayout_2;
    QFrame *searchFM;
    QHBoxLayout *horizontalLayout;
    QToolButton *searchClosePB;
    QLabel *searchLB;
    QLineEdit *searchLE;
    QPushButton *nextPB;
    QPushButton *prevPB;
    QDialogButtonBox *buttonBox;

    void setupUi(QWidget *Snippets)
    {
        if (Snippets->objectName().isEmpty())
            Snippets->setObjectName("Snippets");
        Snippets->resize(640, 400);
        verticalLayout_2 = new QVBoxLayout(Snippets);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        browserw = new QWidget(Snippets);
        browserw->setObjectName("browserw");

        verticalLayout->addWidget(browserw);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        searchFM = new QFrame(Snippets);
        searchFM->setObjectName("searchFM");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(searchFM->sizePolicy().hasHeightForWidth());
        searchFM->setSizePolicy(sizePolicy);
        searchFM->setFrameShape(QFrame::StyledPanel);
        searchFM->setFrameShadow(QFrame::Raised);
        horizontalLayout = new QHBoxLayout(searchFM);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(-1, 1, -1, 1);
        searchClosePB = new QToolButton(searchFM);
        searchClosePB->setObjectName("searchClosePB");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(searchClosePB->sizePolicy().hasHeightForWidth());
        searchClosePB->setSizePolicy(sizePolicy1);
        searchClosePB->setText(QString::fromUtf8("X"));
        searchClosePB->setToolButtonStyle(Qt::ToolButtonTextOnly);

        horizontalLayout->addWidget(searchClosePB);

        searchLB = new QLabel(searchFM);
        searchLB->setObjectName("searchLB");

        horizontalLayout->addWidget(searchLB);

        searchLE = new QLineEdit(searchFM);
        searchLE->setObjectName("searchLE");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(searchLE->sizePolicy().hasHeightForWidth());
        searchLE->setSizePolicy(sizePolicy2);

        horizontalLayout->addWidget(searchLE);

        nextPB = new QPushButton(searchFM);
        nextPB->setObjectName("nextPB");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(nextPB->sizePolicy().hasHeightForWidth());
        nextPB->setSizePolicy(sizePolicy3);

        horizontalLayout->addWidget(nextPB);

        prevPB = new QPushButton(searchFM);
        prevPB->setObjectName("prevPB");
        sizePolicy3.setHeightForWidth(prevPB->sizePolicy().hasHeightForWidth());
        prevPB->setSizePolicy(sizePolicy3);

        horizontalLayout->addWidget(prevPB);


        horizontalLayout_2->addWidget(searchFM);

        buttonBox = new QDialogButtonBox(Snippets);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        horizontalLayout_2->addWidget(buttonBox);


        verticalLayout->addLayout(horizontalLayout_2);


        verticalLayout_2->addLayout(verticalLayout);


        retranslateUi(Snippets);
        QObject::connect(searchClosePB, &QToolButton::clicked, searchFM, qOverload<>(&QFrame::hide));

        QMetaObject::connectSlotsByName(Snippets);
    } // setupUi

    void retranslateUi(QWidget *Snippets)
    {
        Snippets->setWindowTitle(QCoreApplication::translate("Snippets", "Snippets", nullptr));
        searchLB->setText(QCoreApplication::translate("Snippets", "Find:", nullptr));
        nextPB->setText(QCoreApplication::translate("Snippets", "Next", nullptr));
        prevPB->setText(QCoreApplication::translate("Snippets", "Prev", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Snippets: public Ui_Snippets {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SNIPPETS_H
