/********************************************************************************
** Form generated from reading UI file 'configswitch.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGSWITCH_H
#define UI_CONFIGSWITCH_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ConfigSwitchDLG
{
public:
    QVBoxLayout *verticalLayout_2;
    QVBoxLayout *verticalLayout;
    QComboBox *dirsCMB;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *ConfigSwitchDLG)
    {
        if (ConfigSwitchDLG->objectName().isEmpty())
            ConfigSwitchDLG->setObjectName("ConfigSwitchDLG");
        ConfigSwitchDLG->resize(291, 78);
        verticalLayout_2 = new QVBoxLayout(ConfigSwitchDLG);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        dirsCMB = new QComboBox(ConfigSwitchDLG);
        dirsCMB->setObjectName("dirsCMB");

        verticalLayout->addWidget(dirsCMB);

        buttonBox = new QDialogButtonBox(ConfigSwitchDLG);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        verticalLayout_2->addLayout(verticalLayout);


        retranslateUi(ConfigSwitchDLG);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, ConfigSwitchDLG, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, ConfigSwitchDLG, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(ConfigSwitchDLG);
    } // setupUi

    void retranslateUi(QDialog *ConfigSwitchDLG)
    {
        ConfigSwitchDLG->setWindowTitle(QCoreApplication::translate("ConfigSwitchDLG", "Dialog", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ConfigSwitchDLG: public Ui_ConfigSwitchDLG {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONFIGSWITCH_H
