/****************************************************************************
** Meta object code from reading C++ file 'advsearch_w.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../qtgui/advsearch_w.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'advsearch_w.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9AdvSearchE_t {};
} // unnamed namespace

template <> constexpr inline auto AdvSearch::qt_create_metaobjectdata<qt_meta_tag_ZN9AdvSearchE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AdvSearch",
        "startSearch",
        "",
        "std::shared_ptr<Rcl::SearchData>",
        "setDescription",
        "delFiltypPB_clicked",
        "delAFiltypPB_clicked",
        "addFiltypPB_clicked",
        "addAFiltypPB_clicked",
        "guiListsToIgnTypes",
        "filterDatesCB_toggled",
        "filterBirthDatesCB_toggled",
        "filterSizesCB_toggled",
        "restrictFtCB_toggled",
        "restrictCtCB_toggled",
        "runSearch",
        "fromSearch",
        "sdata",
        "browsePB_clicked",
        "saveFileTypes",
        "delClause",
        "updsaved",
        "addClause",
        "slotHistoryNext",
        "slotHistoryPrev",
        "onNewShortcuts"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'startSearch'
        QtMocHelpers::SignalData<void(std::shared_ptr<Rcl::SearchData>, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { QMetaType::Bool, 2 },
        }}),
        // Signal 'setDescription'
        QtMocHelpers::SignalData<void(QString)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'delFiltypPB_clicked'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'delAFiltypPB_clicked'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addFiltypPB_clicked'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addAFiltypPB_clicked'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'guiListsToIgnTypes'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'filterDatesCB_toggled'
        QtMocHelpers::SlotData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'filterBirthDatesCB_toggled'
        QtMocHelpers::SlotData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'filterSizesCB_toggled'
        QtMocHelpers::SlotData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'restrictFtCB_toggled'
        QtMocHelpers::SlotData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'restrictCtCB_toggled'
        QtMocHelpers::SlotData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'runSearch'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fromSearch'
        QtMocHelpers::SlotData<void(std::shared_ptr<Rcl::SearchData>)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 17 },
        }}),
        // Slot 'browsePB_clicked'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveFileTypes'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'delClause'
        QtMocHelpers::SlotData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 21 },
        }}),
        // Slot 'delClause'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'addClause'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 21 },
        }}),
        // Slot 'addClause'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'addClause'
        QtMocHelpers::SlotData<void(int, bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { QMetaType::Bool, 21 },
        }}),
        // Slot 'addClause'
        QtMocHelpers::SlotData<void(int)>(22, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'slotHistoryNext'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHistoryPrev'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onNewShortcuts'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AdvSearch, qt_meta_tag_ZN9AdvSearchE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AdvSearch::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9AdvSearchE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9AdvSearchE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9AdvSearchE_t>.metaTypes,
    nullptr
} };

void AdvSearch::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AdvSearch *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->startSearch((*reinterpret_cast<std::add_pointer_t<std::shared_ptr<Rcl::SearchData>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 1: _t->setDescription((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->delFiltypPB_clicked(); break;
        case 3: _t->delAFiltypPB_clicked(); break;
        case 4: _t->addFiltypPB_clicked(); break;
        case 5: _t->addAFiltypPB_clicked(); break;
        case 6: _t->guiListsToIgnTypes(); break;
        case 7: _t->filterDatesCB_toggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->filterBirthDatesCB_toggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->filterSizesCB_toggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->restrictFtCB_toggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->restrictCtCB_toggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->runSearch(); break;
        case 13: _t->fromSearch((*reinterpret_cast<std::add_pointer_t<std::shared_ptr<Rcl::SearchData>>>(_a[1]))); break;
        case 14: _t->browsePB_clicked(); break;
        case 15: _t->saveFileTypes(); break;
        case 16: _t->delClause((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->delClause(); break;
        case 18: _t->addClause((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: _t->addClause(); break;
        case 20: _t->addClause((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 21: _t->addClause((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 22: _t->slotHistoryNext(); break;
        case 23: _t->slotHistoryPrev(); break;
        case 24: _t->onNewShortcuts(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AdvSearch::*)(std::shared_ptr<Rcl::SearchData> , bool )>(_a, &AdvSearch::startSearch, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AdvSearch::*)(QString )>(_a, &AdvSearch::setDescription, 1))
            return;
    }
}

const QMetaObject *AdvSearch::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AdvSearch::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9AdvSearchE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::AdvSearchBase"))
        return static_cast< Ui::AdvSearchBase*>(this);
    return QDialog::qt_metacast(_clname);
}

int AdvSearch::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 25;
    }
    return _id;
}

// SIGNAL 0
void AdvSearch::startSearch(std::shared_ptr<Rcl::SearchData> _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void AdvSearch::setDescription(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
