/********************************************************************************
** Form generated from reading UI file 'advsearch.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ADVSEARCH_H
#define UI_ADVSEARCH_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDateEdit>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_AdvSearchBase
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget;
    QWidget *tab_5;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *hboxLayout;
    QVBoxLayout *vboxLayout;
    QLabel *textLabel2;
    QComboBox *conjunctCMB;
    QSpacerItem *spacer2;
    QPushButton *delClausePB;
    QPushButton *addClausePB;
    QFrame *clauseFRM;
    QVBoxLayout *vboxLayout1;
    QFrame *clauseline;
    QWidget *tab_6;
    QGridLayout *gridLayout;
    QHBoxLayout *horizontalLayout_2;
    QCheckBox *filterDatesCB;
    QLabel *label;
    QDateEdit *minDateDTE;
    QLabel *label_2;
    QDateEdit *maxDateDTE;
    QFrame *line2;
    QHBoxLayout *horizontalLayout_4;
    QCheckBox *filterBirthDatesCB;
    QLabel *label_birth_from;
    QDateEdit *minBirthDateDTE;
    QLabel *label_birth_to;
    QDateEdit *maxBirthDateDTE;
    QFrame *line_birth;
    QHBoxLayout *horizontalLayout;
    QCheckBox *filterSizesCB;
    QLabel *labelminsize;
    QLineEdit *minSizeLE;
    QLabel *labelmaxsize;
    QLineEdit *maxSizeLE;
    QFrame *line4;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *hboxLayout1;
    QCheckBox *restrictFtCB;
    QCheckBox *restrictCtCB;
    QPushButton *saveFileTypesPB;
    QHBoxLayout *hboxLayout2;
    QVBoxLayout *vboxLayout2;
    QLabel *textLabel2_2;
    QListWidget *yesFiltypsLB;
    QVBoxLayout *vboxLayout3;
    QPushButton *delAFiltypPB;
    QPushButton *delFiltypPB;
    QPushButton *addFiltypPB;
    QPushButton *addAFiltypPB;
    QVBoxLayout *vboxLayout4;
    QLabel *textLabel3;
    QListWidget *noFiltypsLB;
    QFrame *line5;
    QGridLayout *gridLayout1;
    QComboBox *subtreeCMB;
    QPushButton *browsePB;
    QLabel *textLabel1;
    QCheckBox *direxclCB;
    QHBoxLayout *hboxLayout3;
    QPushButton *searchPB;
    QPushButton *dismissPB;

    void setupUi(QDialog *AdvSearchBase)
    {
        if (AdvSearchBase->objectName().isEmpty())
            AdvSearchBase->setObjectName("AdvSearchBase");
        AdvSearchBase->resize(544, 536);
        AdvSearchBase->setSizeGripEnabled(true);
        verticalLayout = new QVBoxLayout(AdvSearchBase);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        tabWidget = new QTabWidget(AdvSearchBase);
        tabWidget->setObjectName("tabWidget");
        tab_5 = new QWidget();
        tab_5->setObjectName("tab_5");
        verticalLayout_2 = new QVBoxLayout(tab_5);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        vboxLayout = new QVBoxLayout();
        vboxLayout->setSpacing(2);
        vboxLayout->setObjectName("vboxLayout");
        textLabel2 = new QLabel(tab_5);
        textLabel2->setObjectName("textLabel2");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(textLabel2->sizePolicy().hasHeightForWidth());
        textLabel2->setSizePolicy(sizePolicy);
        textLabel2->setFrameShape(QFrame::NoFrame);
        textLabel2->setFrameShadow(QFrame::Plain);
        textLabel2->setWordWrap(false);
        textLabel2->setMargin(4);

        vboxLayout->addWidget(textLabel2);

        conjunctCMB = new QComboBox(tab_5);
        conjunctCMB->setObjectName("conjunctCMB");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(8);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(conjunctCMB->sizePolicy().hasHeightForWidth());
        conjunctCMB->setSizePolicy(sizePolicy1);

        vboxLayout->addWidget(conjunctCMB);

        spacer2 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer2);

        delClausePB = new QPushButton(tab_5);
        delClausePB->setObjectName("delClausePB");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(delClausePB->sizePolicy().hasHeightForWidth());
        delClausePB->setSizePolicy(sizePolicy2);
        delClausePB->setAutoDefault(false);

        vboxLayout->addWidget(delClausePB);

        addClausePB = new QPushButton(tab_5);
        addClausePB->setObjectName("addClausePB");
        sizePolicy2.setHeightForWidth(addClausePB->sizePolicy().hasHeightForWidth());
        addClausePB->setSizePolicy(sizePolicy2);
        addClausePB->setAutoDefault(false);

        vboxLayout->addWidget(addClausePB);


        hboxLayout->addLayout(vboxLayout);

        clauseFRM = new QFrame(tab_5);
        clauseFRM->setObjectName("clauseFRM");
        clauseFRM->setFrameShape(QFrame::StyledPanel);
        clauseFRM->setFrameShadow(QFrame::Plain);
        vboxLayout1 = new QVBoxLayout(clauseFRM);
        vboxLayout1->setSpacing(1);
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        clauseline = new QFrame(clauseFRM);
        clauseline->setObjectName("clauseline");
        clauseline->setFrameShape(QFrame::HLine);
        clauseline->setFrameShadow(QFrame::Plain);

        vboxLayout1->addWidget(clauseline);


        hboxLayout->addWidget(clauseFRM);


        verticalLayout_2->addLayout(hboxLayout);

        tabWidget->addTab(tab_5, QString());
        tab_6 = new QWidget();
        tab_6->setObjectName("tab_6");
        gridLayout = new QGridLayout(tab_6);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        filterDatesCB = new QCheckBox(tab_6);
        filterDatesCB->setObjectName("filterDatesCB");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(1);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(filterDatesCB->sizePolicy().hasHeightForWidth());
        filterDatesCB->setSizePolicy(sizePolicy3);

        horizontalLayout_2->addWidget(filterDatesCB);

        label = new QLabel(tab_6);
        label->setObjectName("label");

        horizontalLayout_2->addWidget(label);

        minDateDTE = new QDateEdit(tab_6);
        minDateDTE->setObjectName("minDateDTE");
        minDateDTE->setEnabled(false);

        horizontalLayout_2->addWidget(minDateDTE);

        label_2 = new QLabel(tab_6);
        label_2->setObjectName("label_2");

        horizontalLayout_2->addWidget(label_2);

        maxDateDTE = new QDateEdit(tab_6);
        maxDateDTE->setObjectName("maxDateDTE");
        maxDateDTE->setEnabled(false);

        horizontalLayout_2->addWidget(maxDateDTE);


        gridLayout->addLayout(horizontalLayout_2, 0, 0, 1, 1);

        line2 = new QFrame(tab_6);
        line2->setObjectName("line2");
        line2->setFrameShape(QFrame::HLine);
        line2->setFrameShadow(QFrame::Sunken);

        gridLayout->addWidget(line2, 1, 0, 1, 1);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setSpacing(6);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        filterBirthDatesCB = new QCheckBox(tab_6);
        filterBirthDatesCB->setObjectName("filterBirthDatesCB");
        sizePolicy3.setHeightForWidth(filterBirthDatesCB->sizePolicy().hasHeightForWidth());
        filterBirthDatesCB->setSizePolicy(sizePolicy3);

        horizontalLayout_4->addWidget(filterBirthDatesCB);

        label_birth_from = new QLabel(tab_6);
        label_birth_from->setObjectName("label_birth_from");

        horizontalLayout_4->addWidget(label_birth_from);

        minBirthDateDTE = new QDateEdit(tab_6);
        minBirthDateDTE->setObjectName("minBirthDateDTE");
        minBirthDateDTE->setEnabled(false);

        horizontalLayout_4->addWidget(minBirthDateDTE);

        label_birth_to = new QLabel(tab_6);
        label_birth_to->setObjectName("label_birth_to");

        horizontalLayout_4->addWidget(label_birth_to);

        maxBirthDateDTE = new QDateEdit(tab_6);
        maxBirthDateDTE->setObjectName("maxBirthDateDTE");
        maxBirthDateDTE->setEnabled(false);

        horizontalLayout_4->addWidget(maxBirthDateDTE);


        gridLayout->addLayout(horizontalLayout_4, 2, 0, 1, 1);

        line_birth = new QFrame(tab_6);
        line_birth->setObjectName("line_birth");
        line_birth->setFrameShape(QFrame::HLine);
        line_birth->setFrameShadow(QFrame::Sunken);

        gridLayout->addWidget(line_birth, 3, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        filterSizesCB = new QCheckBox(tab_6);
        filterSizesCB->setObjectName("filterSizesCB");
        sizePolicy3.setHeightForWidth(filterSizesCB->sizePolicy().hasHeightForWidth());
        filterSizesCB->setSizePolicy(sizePolicy3);

        horizontalLayout->addWidget(filterSizesCB);

        labelminsize = new QLabel(tab_6);
        labelminsize->setObjectName("labelminsize");

        horizontalLayout->addWidget(labelminsize);

        minSizeLE = new QLineEdit(tab_6);
        minSizeLE->setObjectName("minSizeLE");
        minSizeLE->setEnabled(false);

        horizontalLayout->addWidget(minSizeLE);

        labelmaxsize = new QLabel(tab_6);
        labelmaxsize->setObjectName("labelmaxsize");

        horizontalLayout->addWidget(labelmaxsize);

        maxSizeLE = new QLineEdit(tab_6);
        maxSizeLE->setObjectName("maxSizeLE");
        maxSizeLE->setEnabled(false);

        horizontalLayout->addWidget(maxSizeLE);


        gridLayout->addLayout(horizontalLayout, 4, 0, 1, 1);

        line4 = new QFrame(tab_6);
        line4->setObjectName("line4");
        line4->setFrameShape(QFrame::HLine);
        line4->setFrameShadow(QFrame::Sunken);

        gridLayout->addWidget(line4, 5, 0, 1, 1);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setObjectName("verticalLayout_3");
        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        restrictFtCB = new QCheckBox(tab_6);
        restrictFtCB->setObjectName("restrictFtCB");
        sizePolicy3.setHeightForWidth(restrictFtCB->sizePolicy().hasHeightForWidth());
        restrictFtCB->setSizePolicy(sizePolicy3);

        hboxLayout1->addWidget(restrictFtCB);

        restrictCtCB = new QCheckBox(tab_6);
        restrictCtCB->setObjectName("restrictCtCB");
        restrictCtCB->setEnabled(false);
        sizePolicy3.setHeightForWidth(restrictCtCB->sizePolicy().hasHeightForWidth());
        restrictCtCB->setSizePolicy(sizePolicy3);

        hboxLayout1->addWidget(restrictCtCB);

        saveFileTypesPB = new QPushButton(tab_6);
        saveFileTypesPB->setObjectName("saveFileTypesPB");
        saveFileTypesPB->setEnabled(false);
        saveFileTypesPB->setAutoDefault(false);

        hboxLayout1->addWidget(saveFileTypesPB);


        verticalLayout_3->addLayout(hboxLayout1);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName("hboxLayout2");
        vboxLayout2 = new QVBoxLayout();
        vboxLayout2->setSpacing(0);
        vboxLayout2->setObjectName("vboxLayout2");
        textLabel2_2 = new QLabel(tab_6);
        textLabel2_2->setObjectName("textLabel2_2");
        textLabel2_2->setWordWrap(false);

        vboxLayout2->addWidget(textLabel2_2);

        yesFiltypsLB = new QListWidget(tab_6);
        yesFiltypsLB->setObjectName("yesFiltypsLB");
        yesFiltypsLB->setEnabled(false);
        yesFiltypsLB->setMinimumSize(QSize(200, 20));
        yesFiltypsLB->setSelectionMode(QAbstractItemView::ExtendedSelection);

        vboxLayout2->addWidget(yesFiltypsLB);


        hboxLayout2->addLayout(vboxLayout2);

        vboxLayout3 = new QVBoxLayout();
        vboxLayout3->setSpacing(0);
        vboxLayout3->setObjectName("vboxLayout3");
        delAFiltypPB = new QPushButton(tab_6);
        delAFiltypPB->setObjectName("delAFiltypPB");
        delAFiltypPB->setEnabled(false);
        delAFiltypPB->setAutoDefault(false);

        vboxLayout3->addWidget(delAFiltypPB);

        delFiltypPB = new QPushButton(tab_6);
        delFiltypPB->setObjectName("delFiltypPB");
        delFiltypPB->setEnabled(false);
        delFiltypPB->setAutoDefault(false);

        vboxLayout3->addWidget(delFiltypPB);

        addFiltypPB = new QPushButton(tab_6);
        addFiltypPB->setObjectName("addFiltypPB");
        addFiltypPB->setEnabled(false);
        addFiltypPB->setAutoDefault(false);

        vboxLayout3->addWidget(addFiltypPB);

        addAFiltypPB = new QPushButton(tab_6);
        addAFiltypPB->setObjectName("addAFiltypPB");
        addAFiltypPB->setEnabled(false);
        addAFiltypPB->setAutoDefault(false);

        vboxLayout3->addWidget(addAFiltypPB);


        hboxLayout2->addLayout(vboxLayout3);

        vboxLayout4 = new QVBoxLayout();
        vboxLayout4->setSpacing(0);
        vboxLayout4->setObjectName("vboxLayout4");
        textLabel3 = new QLabel(tab_6);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        vboxLayout4->addWidget(textLabel3);

        noFiltypsLB = new QListWidget(tab_6);
        noFiltypsLB->setObjectName("noFiltypsLB");
        noFiltypsLB->setEnabled(false);
        noFiltypsLB->setMinimumSize(QSize(200, 20));
        noFiltypsLB->setSelectionMode(QAbstractItemView::ExtendedSelection);

        vboxLayout4->addWidget(noFiltypsLB);


        hboxLayout2->addLayout(vboxLayout4);


        verticalLayout_3->addLayout(hboxLayout2);


        gridLayout->addLayout(verticalLayout_3, 6, 0, 1, 1);

        line5 = new QFrame(tab_6);
        line5->setObjectName("line5");
        line5->setFrameShape(QFrame::HLine);
        line5->setFrameShadow(QFrame::Sunken);

        gridLayout->addWidget(line5, 7, 0, 1, 1);

        gridLayout1 = new QGridLayout();
        gridLayout1->setSpacing(6);
        gridLayout1->setObjectName("gridLayout1");
        subtreeCMB = new QComboBox(tab_6);
        subtreeCMB->setObjectName("subtreeCMB");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy4.setHorizontalStretch(8);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(subtreeCMB->sizePolicy().hasHeightForWidth());
        subtreeCMB->setSizePolicy(sizePolicy4);
        subtreeCMB->setMinimumSize(QSize(300, 0));
        subtreeCMB->setEditable(true);
        subtreeCMB->setMaxCount(20);
        subtreeCMB->setInsertPolicy(QComboBox::NoInsert);
        subtreeCMB->setDuplicatesEnabled(false);

        gridLayout1->addWidget(subtreeCMB, 1, 0, 1, 1);

        browsePB = new QPushButton(tab_6);
        browsePB->setObjectName("browsePB");
        browsePB->setAutoDefault(false);

        gridLayout1->addWidget(browsePB, 1, 1, 1, 1);

        textLabel1 = new QLabel(tab_6);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout1->addWidget(textLabel1, 0, 0, 1, 1);

        direxclCB = new QCheckBox(tab_6);
        direxclCB->setObjectName("direxclCB");

        gridLayout1->addWidget(direxclCB, 0, 1, 1, 1);


        gridLayout->addLayout(gridLayout1, 8, 0, 1, 1);

        tabWidget->addTab(tab_6, QString());

        verticalLayout->addWidget(tabWidget);

        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setSpacing(6);
        hboxLayout3->setObjectName("hboxLayout3");
        searchPB = new QPushButton(AdvSearchBase);
        searchPB->setObjectName("searchPB");

        hboxLayout3->addWidget(searchPB);

        dismissPB = new QPushButton(AdvSearchBase);
        dismissPB->setObjectName("dismissPB");
        dismissPB->setAutoDefault(false);

        hboxLayout3->addWidget(dismissPB);


        verticalLayout->addLayout(hboxLayout3);


        retranslateUi(AdvSearchBase);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(AdvSearchBase);
    } // setupUi

    void retranslateUi(QDialog *AdvSearchBase)
    {
        AdvSearchBase->setWindowTitle(QCoreApplication::translate("AdvSearchBase", "Advanced search", nullptr));
#if QT_CONFIG(tooltip)
        textLabel2->setToolTip(QCoreApplication::translate("AdvSearchBase", "All non empty fields on the right will be combined with AND (\"All clauses\" choice) or OR (\"Any clause\" choice) conjunctions. <br>\"Any\" \"All\" and \"None\" field types can accept a mix of simple words, and phrases enclosed in double quotes.<br>Fields with no data are ignored.", nullptr));
#endif // QT_CONFIG(tooltip)
        textLabel2->setText(QCoreApplication::translate("AdvSearchBase", "Search for <br>documents<br>satisfying:", nullptr));
#if QT_CONFIG(tooltip)
        conjunctCMB->setToolTip(QCoreApplication::translate("AdvSearchBase", "All non empty fields on the right will be combined with AND (\"All clauses\" choice) or OR (\"Any clause\" choice) conjunctions. <br>\"Any\" \"All\" and \"None\" field types can accept a mix of simple words, and phrases enclosed in double quotes.<br>Fields with no data are ignored.", nullptr));
#endif // QT_CONFIG(tooltip)
        delClausePB->setText(QCoreApplication::translate("AdvSearchBase", "Delete clause", nullptr));
        addClausePB->setText(QCoreApplication::translate("AdvSearchBase", "Add clause", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_5), QCoreApplication::translate("AdvSearchBase", "Find", nullptr));
#if QT_CONFIG(tooltip)
        filterDatesCB->setToolTip(QCoreApplication::translate("AdvSearchBase", "Check this to enable filtering on dates", nullptr));
#endif // QT_CONFIG(tooltip)
        filterDatesCB->setText(QCoreApplication::translate("AdvSearchBase", "Filter dates", nullptr));
        label->setText(QCoreApplication::translate("AdvSearchBase", "From", nullptr));
        label_2->setText(QCoreApplication::translate("AdvSearchBase", "To", nullptr));
#if QT_CONFIG(tooltip)
        filterBirthDatesCB->setToolTip(QCoreApplication::translate("AdvSearchBase", "Check this to enable filtering on dates", nullptr));
#endif // QT_CONFIG(tooltip)
        filterBirthDatesCB->setText(QCoreApplication::translate("AdvSearchBase", "Filter birth dates", nullptr));
        label_birth_from->setText(QCoreApplication::translate("AdvSearchBase", "From", nullptr));
        label_birth_to->setText(QCoreApplication::translate("AdvSearchBase", "To", nullptr));
#if QT_CONFIG(tooltip)
        filterSizesCB->setToolTip(QCoreApplication::translate("AdvSearchBase", "Check this to enable filtering on sizes", nullptr));
#endif // QT_CONFIG(tooltip)
        filterSizesCB->setText(QCoreApplication::translate("AdvSearchBase", "Filter sizes", nullptr));
#if QT_CONFIG(tooltip)
        labelminsize->setToolTip(QCoreApplication::translate("AdvSearchBase", "Minimum size. You can use k/K,m/M,g/G as multipliers", nullptr));
#endif // QT_CONFIG(tooltip)
        labelminsize->setText(QCoreApplication::translate("AdvSearchBase", "Min. Size", nullptr));
#if QT_CONFIG(tooltip)
        labelmaxsize->setToolTip(QCoreApplication::translate("AdvSearchBase", "Maximum size. You can use k/K,m/M,g/G as multipliers", nullptr));
#endif // QT_CONFIG(tooltip)
        labelmaxsize->setText(QCoreApplication::translate("AdvSearchBase", "Max. Size", nullptr));
#if QT_CONFIG(tooltip)
        restrictFtCB->setToolTip(QCoreApplication::translate("AdvSearchBase", "Check this to enable filtering on file types", nullptr));
#endif // QT_CONFIG(tooltip)
        restrictFtCB->setText(QCoreApplication::translate("AdvSearchBase", "Restrict file types", nullptr));
#if QT_CONFIG(tooltip)
        restrictCtCB->setToolTip(QCoreApplication::translate("AdvSearchBase", "Check this to use file categories instead of raw mime types", nullptr));
#endif // QT_CONFIG(tooltip)
        restrictCtCB->setText(QCoreApplication::translate("AdvSearchBase", "By categories", nullptr));
        saveFileTypesPB->setText(QCoreApplication::translate("AdvSearchBase", "Save as default", nullptr));
        textLabel2_2->setText(QCoreApplication::translate("AdvSearchBase", "Searched file types", nullptr));
        delAFiltypPB->setText(QCoreApplication::translate("AdvSearchBase", "All ---->", nullptr));
        delFiltypPB->setText(QCoreApplication::translate("AdvSearchBase", "Sel ----->", nullptr));
        addFiltypPB->setText(QCoreApplication::translate("AdvSearchBase", "<----- Sel", nullptr));
        addAFiltypPB->setText(QCoreApplication::translate("AdvSearchBase", "<----- All", nullptr));
        textLabel3->setText(QCoreApplication::translate("AdvSearchBase", "Ignored file types", nullptr));
#if QT_CONFIG(tooltip)
        subtreeCMB->setToolTip(QCoreApplication::translate("AdvSearchBase", "Enter top directory for search", nullptr));
#endif // QT_CONFIG(tooltip)
        browsePB->setText(QCoreApplication::translate("AdvSearchBase", "Browse", nullptr));
        textLabel1->setText(QCoreApplication::translate("AdvSearchBase", "Restrict results to files in subtree:", nullptr));
        direxclCB->setText(QCoreApplication::translate("AdvSearchBase", "Invert", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_6), QCoreApplication::translate("AdvSearchBase", "Filter", nullptr));
        searchPB->setText(QCoreApplication::translate("AdvSearchBase", "Start Search", nullptr));
        dismissPB->setText(QCoreApplication::translate("AdvSearchBase", "Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class AdvSearchBase: public Ui_AdvSearchBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ADVSEARCH_H
