/********************************************************************************
** Form generated from reading UI file 'ptrans.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PTRANS_H
#define UI_PTRANS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_PTransEditBase
{
public:
    QVBoxLayout *verticalLayout_2;
    QVBoxLayout *verticalLayout;
    QComboBox *whatIdxCMB;
    QTableWidget *transTW;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *addPB;
    QPushButton *delPB;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *cancelPB;
    QPushButton *savePB;

    void setupUi(QDialog *EditTransBase)
    {
        if (EditTransBase->objectName().isEmpty())
            EditTransBase->setObjectName("EditTransBase");
        EditTransBase->resize(649, 362);
        verticalLayout_2 = new QVBoxLayout(EditTransBase);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(6);
        verticalLayout->setObjectName("verticalLayout");
        whatIdxCMB = new QComboBox(EditTransBase);
        whatIdxCMB->setObjectName("whatIdxCMB");

        verticalLayout->addWidget(whatIdxCMB);

        transTW = new QTableWidget(EditTransBase);
        if (transTW->columnCount() < 2)
            transTW->setColumnCount(2);
        transTW->setObjectName("transTW");
        transTW->setFrameShape(QFrame::StyledPanel);
        transTW->setFrameShadow(QFrame::Sunken);
        transTW->setEditTriggers(QAbstractItemView::NoEditTriggers);
        transTW->setSelectionMode(QAbstractItemView::ExtendedSelection);
        transTW->setSelectionBehavior(QAbstractItemView::SelectRows);
        transTW->setShowGrid(true);
        transTW->setSortingEnabled(true);
        transTW->setColumnCount(2);
        transTW->setProperty("allColumnsShowFocus", QVariant(true));
        transTW->setProperty("showSortIndicator", QVariant(false));
        transTW->horizontalHeader()->setMinimumSectionSize(20);
        transTW->horizontalHeader()->setDefaultSectionSize(300);
        transTW->horizontalHeader()->setProperty("showSortIndicator", QVariant(false));
        transTW->horizontalHeader()->setStretchLastSection(true);
        transTW->verticalHeader()->setVisible(false);

        verticalLayout->addWidget(transTW);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        addPB = new QPushButton(EditTransBase);
        addPB->setObjectName("addPB");

        horizontalLayout_2->addWidget(addPB);

        delPB = new QPushButton(EditTransBase);
        delPB->setObjectName("delPB");
        delPB->setEnabled(false);

        horizontalLayout_2->addWidget(delPB);


        verticalLayout->addLayout(horizontalLayout_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        cancelPB = new QPushButton(EditTransBase);
        cancelPB->setObjectName("cancelPB");

        horizontalLayout->addWidget(cancelPB);

        savePB = new QPushButton(EditTransBase);
        savePB->setObjectName("savePB");

        horizontalLayout->addWidget(savePB);


        verticalLayout->addLayout(horizontalLayout);


        verticalLayout_2->addLayout(verticalLayout);


        retranslateUi(EditTransBase);

        QMetaObject::connectSlotsByName(EditTransBase);
    } // setupUi

    void retranslateUi(QDialog *EditTransBase)
    {
        EditTransBase->setWindowTitle(QCoreApplication::translate("PTransEditBase", "Path Translations", nullptr));
#if QT_CONFIG(tooltip)
        transTW->setToolTip(QCoreApplication::translate("PTransEditBase", "Select one or several file types, then use the controls in the frame below to change how they are processed", nullptr));
#endif // QT_CONFIG(tooltip)
        addPB->setText(QCoreApplication::translate("PTransEditBase", "Add", nullptr));
        delPB->setText(QCoreApplication::translate("PTransEditBase", "Delete", nullptr));
        cancelPB->setText(QCoreApplication::translate("PTransEditBase", "Cancel", nullptr));
        savePB->setText(QCoreApplication::translate("PTransEditBase", "Save", nullptr));
    } // retranslateUi

};

namespace Ui {
    class PTransEditBase: public Ui_PTransEditBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PTRANS_H
