/********************************************************************************
** Form generated from reading UI file 'viewaction.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_VIEWACTION_H
#define UI_VIEWACTION_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ViewActionBase
{
public:
    QVBoxLayout *verticalLayout_3;
    QLabel *textLabel1;
    QCheckBox *useDesktopCB;
    QTableWidget *actionsLV;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_2;
    QLabel *currentLBL;
    QPushButton *selSamePB;
    QFrame *frame;
    QVBoxLayout *verticalLayout_2;
    QVBoxLayout *verticalLayout;
    QLabel *label_new;
    QCheckBox *setExceptCB;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label;
    QLineEdit *newActionLE;
    QPushButton *chgActPB;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *closePB;

    void setupUi(QDialog *ViewActionBase)
    {
        if (ViewActionBase->objectName().isEmpty())
            ViewActionBase->setObjectName("ViewActionBase");
        ViewActionBase->resize(640, 480);
        verticalLayout_3 = new QVBoxLayout(ViewActionBase);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        textLabel1 = new QLabel(ViewActionBase);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        verticalLayout_3->addWidget(textLabel1);

        useDesktopCB = new QCheckBox(ViewActionBase);
        useDesktopCB->setObjectName("useDesktopCB");

        verticalLayout_3->addWidget(useDesktopCB);

        actionsLV = new QTableWidget(ViewActionBase);
        if (actionsLV->columnCount() < 2)
            actionsLV->setColumnCount(2);
        actionsLV->setObjectName("actionsLV");
        actionsLV->setFrameShape(QFrame::StyledPanel);
        actionsLV->setFrameShadow(QFrame::Sunken);
        actionsLV->setEditTriggers(QAbstractItemView::NoEditTriggers);
        actionsLV->setSelectionMode(QAbstractItemView::ExtendedSelection);
        actionsLV->setSelectionBehavior(QAbstractItemView::SelectRows);
        actionsLV->setShowGrid(true);
        actionsLV->setSortingEnabled(true);
        actionsLV->setColumnCount(2);
        actionsLV->setProperty("allColumnsShowFocus", QVariant(true));
        actionsLV->setProperty("showSortIndicator", QVariant(true));
        actionsLV->horizontalHeader()->setDefaultSectionSize(350);
        actionsLV->horizontalHeader()->setProperty("showSortIndicator", QVariant(true));
        actionsLV->horizontalHeader()->setStretchLastSection(true);
        actionsLV->verticalHeader()->setVisible(false);

        verticalLayout_3->addWidget(actionsLV);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(6);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_2 = new QLabel(ViewActionBase);
        label_2->setObjectName("label_2");

        horizontalLayout_3->addWidget(label_2);

        currentLBL = new QLabel(ViewActionBase);
        currentLBL->setObjectName("currentLBL");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(currentLBL->sizePolicy().hasHeightForWidth());
        currentLBL->setSizePolicy(sizePolicy);
        currentLBL->setFrameShape(QFrame::Box);
        currentLBL->setFrameShadow(QFrame::Raised);
        currentLBL->setTextFormat(Qt::PlainText);
        currentLBL->setTextInteractionFlags(Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        horizontalLayout_3->addWidget(currentLBL);

        selSamePB = new QPushButton(ViewActionBase);
        selSamePB->setObjectName("selSamePB");

        horizontalLayout_3->addWidget(selSamePB);


        verticalLayout_3->addLayout(horizontalLayout_3);

        frame = new QFrame(ViewActionBase);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::Box);
        frame->setFrameShadow(QFrame::Plain);
        verticalLayout_2 = new QVBoxLayout(frame);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(6);
        verticalLayout->setObjectName("verticalLayout");
        label_new = new QLabel(frame);
        label_new->setObjectName("label_new");

        verticalLayout->addWidget(label_new);

        setExceptCB = new QCheckBox(frame);
        setExceptCB->setObjectName("setExceptCB");

        verticalLayout->addWidget(setExceptCB);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label = new QLabel(frame);
        label->setObjectName("label");

        horizontalLayout_2->addWidget(label);

        newActionLE = new QLineEdit(frame);
        newActionLE->setObjectName("newActionLE");

        horizontalLayout_2->addWidget(newActionLE);


        verticalLayout->addLayout(horizontalLayout_2);

        chgActPB = new QPushButton(frame);
        chgActPB->setObjectName("chgActPB");

        verticalLayout->addWidget(chgActPB);


        verticalLayout_2->addLayout(verticalLayout);


        verticalLayout_3->addWidget(frame);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        closePB = new QPushButton(ViewActionBase);
        closePB->setObjectName("closePB");

        horizontalLayout->addWidget(closePB);


        verticalLayout_3->addLayout(horizontalLayout);


        retranslateUi(ViewActionBase);

        QMetaObject::connectSlotsByName(ViewActionBase);
    } // setupUi

    void retranslateUi(QDialog *ViewActionBase)
    {
        ViewActionBase->setWindowTitle(QCoreApplication::translate("ViewActionBase", "Native Viewers", nullptr));
        textLabel1->setText(QCoreApplication::translate("ViewActionBase", "Select one or several mime types then use the controls in the bottom frame to change how they are processed.", nullptr));
        useDesktopCB->setText(QCoreApplication::translate("ViewActionBase", "Use Desktop preferences by default", nullptr));
#if QT_CONFIG(tooltip)
        actionsLV->setToolTip(QCoreApplication::translate("ViewActionBase", "Select one or several file types, then use the controls in the frame below to change how they are processed", nullptr));
#endif // QT_CONFIG(tooltip)
        label_2->setText(QCoreApplication::translate("ViewActionBase", "Recoll action:", nullptr));
        currentLBL->setText(QCoreApplication::translate("ViewActionBase", "current value", nullptr));
        selSamePB->setText(QCoreApplication::translate("ViewActionBase", "Select same", nullptr));
        label_new->setText(QCoreApplication::translate("ViewActionBase", "<b>New Values:</b>", nullptr));
        setExceptCB->setText(QCoreApplication::translate("ViewActionBase", "Exception to Desktop preferences", nullptr));
        label->setText(QCoreApplication::translate("ViewActionBase", "Action (empty -> recoll default)", nullptr));
#if QT_CONFIG(tooltip)
        label->setToolTip(QCoreApplication::translate("ViewActionBase", "The value is a command line to be executed. Substitutions: %s: search string; %p: page number; <br>%f: document file name. F1 for more help.", nullptr));
#endif // QT_CONFIG(tooltip)
        chgActPB->setText(QCoreApplication::translate("ViewActionBase", "Apply to current selection", nullptr));
        closePB->setText(QCoreApplication::translate("ViewActionBase", "Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ViewActionBase: public Ui_ViewActionBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_VIEWACTION_H
